enyo.kind({
	name: "WiFiSharing",
	kind: "VFlexBox",
	align: "center",
	width: "100%",
	style: "background-color: black;",
	components: [
		//ApplicationEvents Handler
		{kind: "ApplicationEvents", onLoad: "appLoad", onUnload: "appClose", onBack: "backGesture"},
		
		//Samba Service
		{kind: "SambaService", name: "samba", onPrefsLoaded: "prefsLoaded", onActivated: "sambaActivated", onDeactivated: "sambaDeactivated"},
		
		/*
		//Top Bar
		{kind: "Toolbar", width: "100%"},
		*/
		
		//Main Scene
		{kind: "VFlexBox", flex: 1, pack: "center", align: "center", components: [
			//WiFi Image by Aerotox
			{name: "WiFiImage", kind: "Image", src: "app/images/WiFi.png", style: "opacity: 1.0;"},
			/* Not required- simplicity
			//Status Text
			{content: "Status:", style: "color: white;"},
			{content: "<font size=3>In Development</font>", style: "color: white;"}, //Replace with actual status
			*/
		]},
		
		//Bottom Bar
		{kind: "Toolbar", width: "100%", pack: "end", components: [
			//About Button
			{caption: "About", onclick: "openAbout"},
			{kind: "Spacer"},
			//DEBUG Button
			//{name: "DebugButton", caption: "Debuggery", onclick: "debugButton"},
			//{kind: "Spacer"},
			//Show Preferences - Next Version
			{name: "PrefsButton", caption: "Preferences", onclick: "openPrefs", disabled: true}
		]},
		
		//Preferences Pane
		{name: "Preferences", kind: "PreferencesPane", onBeforeOpen: "setupPrefs", onApplyPressed: "applyPressed"},
		
		//About Popup
		{name: "AboutPopup", kind: "Popup",
		components:[
			{kind: "VFlexBox", align: "center", height: "150px", components: [
				{content: "WiFi File Sharing"},
				{kind: "Spacer"},
				{kind: "HtmlContent", srcId: "aboutContent"},
				{kind: "Spacer"},
				{kind: "Button", content: "Close", onclick: "closeAbout", width: "50%"},
			]},
		]},
		
		//First Use Popup
		{name: "FirstUsePopup", kind: "Popup", width: "720px", height: "460px", //401px to stop cutoff on Veer, this offends my perfectionistic sensibilities.
		components:[
			{kind: "VFlexBox", width: "100%", height: "100%", components: [
				{kind: "Scroller", flex: 1, components: [
					{kind: "VFlexBox", align: "center", pack: "center", components: [
						{kind: "HtmlContent", srcId: "firstUseContent"},
						{kind: "Button", content: "Dismiss", onclick: "closeFirstUse", width: "50%"},
						{height: "20px"},
					]},
				]},
			]},
		]},
	],
	
	//Prefs Pane
	openPrefs: function(inSender) {
		this.$.Preferences.open();
	},
	
	closePrefs: function(inSender) {
		this.$.Preferences.close();
	},
	
	//About Popup
	openAbout: function(inSender) {
		this.$.AboutPopup.openAtCenter();
	},
	
	closeAbout: function(inSender) {
		this.$.AboutPopup.close();
	},
	
	//First Use Popup
	openFirstUse: function(inSender) {
		this.$.FirstUsePopup.openAtCenter();
	},
	
	closeFirstUse: function(inSender) {
		this.$.FirstUsePopup.close();
	},
	
	//On App Load
	appLoad: function(inSender) {
		//Setup global samba
		window.samba = this.$.samba;
		
		//Scan for /media/ext3fs and enable the share accordingly
		enyo.error("Scanning for ext3fs...");
		samba.detectExt3FS();
		
		//Disable Screen Dim
		enyo.error("Disabling Screen Dim...");
		window.PalmSystem.setWindowProperties({ blockScreenTimeout: true });
	
		//Check if localStorage exists
		enyo.error("Checking localStorage for Preferences...");
		if(localStorage.getItem("WiFiSharing.UIPrefs"))
		{
			samba.getPref("global", "netbios name");
			samba.getPref("global", "server string");
			samba.getPref("Public", "available");
			samba.getPref("Public", "writable");
			samba.getPref("Public", "browseable");
			samba.getPref("Internal", "available");
			samba.getPref("Internal", "writable");
			samba.getPref("Internal", "browseable");
			samba.getPref("Root", "available");
			samba.getPref("Root", "writable");
			samba.getPref("Root", "browseable");
			samba.getPref("ext3fs", "available");
			samba.getPref("ext3fs", "writable");
			samba.getPref("ext3fs", "browseable");
			prefStorage.load();
		}
		else
		{
			enyo.error("No preferences found, showing First Use popup.");
			//If not, show the first use window and load in the device name
			this.openFirstUse();
			//Device name- returns undefined in Safari
			var deviceInfo = enyo.fetchDeviceInfo();
			//Setup smb.conf defaults - FIXME:should only have an effect if using unmodified, preconfigured smb.conf
			this.$.samba.setPref("global", "netbios name", deviceInfo.modelNameAscii);
			this.$.samba.setPref("global", "server string", deviceInfo.modelNameAscii + " Samba Share");
			samba.getPref("Public", "available");
			samba.getPref("Public", "writable");
			samba.getPref("Public", "browseable");
			samba.getPref("Internal", "available");
			samba.getPref("Internal", "writable");
			samba.getPref("Internal", "browseable");
			samba.getPref("Root", "available");
			samba.getPref("Root", "writable");
			samba.getPref("Root", "browseable");
			samba.getPref("ext3fs", "available");
			samba.getPref("ext3fs", "writable");
			samba.getPref("ext3fs", "browseable");
			//Save UI Prefs (And thus firstuse)
			prefStorage.save();
		}
		
		//Activate Samba
		enyo.error("Activating Samba...");
		this.$.samba.activate();
	},
	
	appClose: function(inSender) {
		//Disable Samba
		enyo.log("Deactivating Samba...");
		this.$.samba.deactivate();
		//Save UI Prefs
		prefStorage.save();
	},
	
	//Back Gesture
	backGesture: function(inSender, inEvent) {
		//inEvent.stopPropagation();
		//inEvent.preventDefault();
		enyo.stopEvent(inEvent);
		this.closePrefs();
		this.closeAbout();
		this.closeFirstUse();
		//return -1;
	},
	
	//Setup Preferences Elements
	setupPrefs: function(inSender) {
		this.$.Preferences.setup(smbSettings);
	},
	
	//DEBUG Button
	debugButton: function() {
	// Delete Preferences
		enyo.error("Deleting Preferences...");
		localStorage.removeItem("WiFiSharing.UIPrefs");
		enyo.error("Preferences (Allegedly) Deleted.");
	},
	
	//Event Handlers
	//Add visual feedback for samba activation
	sambaActivated: function()
	{
		//this.$.WiFiImage.setStyle("opacity: 1.0;");
	},
	
	sambaDeactivated: function()
	{
		//this.$.WiFiImage.setStyle("opacity: 0.5;");
	},
	
	applyPressed: function()
	{
		//this.$.WiFiImage.setStyle("opacity: 0.5;");
	},
	
	prefsLoaded: function()
	{
		this.$.PrefsButton.setDisabled(false);
		if(smbSettings["global"]["netbios name"] == "DEVICENAME")
		{
			var deviceInfo = enyo.fetchDeviceInfo();
			enyo.error("Default name. BAD.");
			this.$.samba.setPref("global", "netbios name", deviceInfo.modelNameAscii);
			this.$.samba.setPref("global", "server string", deviceInfo.modelNameAscii + " Samba Share");
		}
	},
});

//Preferences Pane
enyo.kind({
	name: "PreferencesPane",
	kind: "Toaster",
	flyInFrom: "right",
	height: "100%",
	style: "position: absolute; top: 0px;",
	events: {
		onApplyPressed: "",
	},
	components: [	
		{kind: "VFlexBox", width: "320px", height: "100%", style: "background-color: #e5e5e5;",
			components: [
				{kind: "Toolbar", content: "Preferences", style: "color: white;"},
				{name: "PrefsContent", kind: "PreferencesDialog", flex: 1},
				{kind: "Toolbar", pack: "end", components:[
					{kind: "GrabButton", onclick: "close"},
					{caption: "Apply", onclick: "applyPrefs"},
				]}
			]
		}
	],
	
	//Setup PrefsContent Elements
	setup: function() {
		this.$.PrefsContent.setup();
	},
	
	applyPrefs: function() {
		this.doApplyPressed();
		samba.activate();
		this.close();
	}
});